@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2019-2024, by Samuel Williams.
+# Copyright, 2019-2025, by Samuel Williams.
 
 def build
 	# Fetch the code:
@@ -30,7 +30,7 @@ def server
 	
 	container = Async::Container.new
 	
-	Console.logger.info(self){"Starting server..."}
+	Console.info(self){"Starting server..."}
 	
 	container.run(count: 1) do
 		server = Async::HTTP::Server.for(endpoint, protocol: Async::HTTP::Protocol::HTTP2, scheme: "https") do |request|
