@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2018-2025, by Samuel Williams.
+# Copyright, 2018-2026, by Samuel Williams.
 # Copyright, 2022, by Marco Concetto Rudilosso.
 # Copyright, 2023, by Thomas Morgan.
 
@@ -36,7 +36,7 @@ module Async
 					
 					attr :input
 					
-					def add_header(key, value)
+					def add_header(key, value, trailer: false)
 						if key == CONNECTION
 							raise ::Protocol::HTTP2::HeaderError, "Connection header is not allowed!"
 						elsif key.start_with? ":"
@@ -44,13 +44,13 @@ module Async
 						elsif key =~ /[A-Z]/
 							raise ::Protocol::HTTP2::HeaderError, "Invalid upper-case characters in header #{key}!"
 						else
-							@headers.add(key, value)
+							@headers.add(key, value, trailer: trailer)
 						end
 					end
 					
 					def receive_trailing_headers(headers, end_stream)
 						headers.each do |key, value|
-							add_header(key, value)
+							add_header(key, value, trailer: true)
 						end
 					end
 					
@@ -64,6 +64,10 @@ module Async
 						if @input and frame.end_stream?
 							@input.close_write
 						end
+					rescue ::Protocol::HTTP::InvalidTrailerError => error
+						Console.warn(self, error)
+						
+						send_reset_stream(::Protocol::HTTP2::Error::PROTOCOL_ERROR)
 					rescue ::Protocol::HTTP2::HeaderError => error
 						Console.debug(self, "Error while processing headers!", error)
 						
